package com.delta.acs.snapp.gates.web.client.lightstreamer.setup;



import com.google.gwt.core.client.JavaScriptObject;

public class LSGWTPushPage {
    /**
     * Opaque JavaScript object containing the PushPage instance.
     */
    private JavaScriptObject page;
    
    /**
     * Lightstreamer client configurations.
     */
    private String domain;
    private String host;
    private String port;
    private String adapterSet;
    private String libPath;
    private String engineName;
    private String engineKind = "SHARE_SESSION";
    private String statusImgDir; 
    private boolean debugAlerts = false;
    private boolean remoteAlerts = false;

    /**
     * This method should be called once the instance is properly configured,
     * it executes the proper javascript so that this instance is ready to
     * accept addTable calls.
     */
    public void start() {
        if (this.page != null) {
            throw new IllegalStateException("LSGWTPushPage already started");
        }
        this.page = this.createPushPage();
        if (this.statusImgDir != null) {
            this.initStatusHandler();
        }
        this.initPushPage();
    }
    
    /**
     * Create the JavaScript Lightstreamer PushPage object.
     */
    private native JavaScriptObject createPushPage() /*-{
        var page = new $wnd.PushPage(); 
        return page;
    }-*/;
   
    /**
     * Applies the configurations of this instance to the Lightstreamer Web Client and start data streaming
     * as soon as the engine is ready for that
     */
    private native void initPushPage() /*-{
        //thisPage is a pointer to the javascript object generated by the createPushPage method and saved by
        //the start method. 
        var thisPage = this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::page;
        
        //the that variable will be used in the onEngine... callbacks to access this instance
        var that = this;

        //needed configuration are applied to the javascript objects
        if (this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::domain != null) {
            thisPage.context.setDomain(this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::domain);
        }
         ///thisPage.context.setDomain(null);
         
//alert(document.domain+"::::Navigator::::"+navigator.appName);
        thisPage.context.setDebugAlertsOnClientError(this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::debugAlerts);
        thisPage.context.setRemoteAlertsOnClientError(this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::remoteAlerts);
       
        //this callback is called once the engine, that makes the connection to the server, is ready to
        //be configured.
        thisPage.onEngineCreation = function(engine) {
            if (!engine) {
                alert("Failed to initialize the Lightstreamer Engine");
                return;
            }
            
            //so the engine is configured
            engine.connection.setAdapterName(that.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::adapterSet);
            engine.connection.setLSHost(that.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::host);
            if (that.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::port) {
                engine.connection.setLSPort(that.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::port);
            }
            
            //and then the connection is started
            engine.changeStatus("STREAMING");
        }
        if (this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::statusImgDir) {
            
            //this callback is called once the engine is ready.
            thisPage.onEngineReady = function(engine) {
            
                // define the callback on the Engine status change and change the image accordingly
//                engine.onStatusChange = function(newStatus) {
//                    var statusImgSrc;
//                    var statusText;
//                    
//                    var masterImgString = thisPage.isMasterPushPage() ? "_master" : "";
//                    var masterTextString = thisPage.isMasterPushPage() ? "<br/> Master page" : "";
//                    
//                    if (newStatus == "STREAMING") {
//                      statusImgSrc = "status_connected_streaming" + masterImgString + ".png";
//                      statusText = "Connected to Push Server (Streaming)" + masterTextString;
//                    } else if (newStatus == "POLLING") {
//                      statusImgSrc = "status_connected_polling" + masterImgString + ".png";
//                      statusText = "Connected to Push Server (Smart polling)" + masterTextString;
//                    } else if (newStatus == "STALLED") {
//                      statusImgSrc = "status_stalled" + masterImgString + ".png";
//                      statusText = "Connection to Push Server stalled" + masterTextString;
//                    } else if (newStatus == "DISCONNECTED") {
//                      statusImgSrc = "status_disconnected" + masterImgString + ".png";
//                      statusText = "Disconnected from Push Server" + masterTextString;
//                    } else if (newStatus == "CONNECTING") {
//                      statusImgSrc = "status_disconnected" + masterImgString + ".png";
//                      statusText = "Trying to connect to Push Server..." + masterTextString;
//                    }
//                    document.statusImg.src = that.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::statusImgDir + statusImgSrc;
//                    document.getElementById("statusDiv").innerHTML = statusText;
//                    // on Safari write to the status bar, to overwrite a misleading download message
//                    if (navigator.vendor && navigator.vendor.indexOf("Apple") != -1) window.status = " ";  
//                    
//                }
            
//                engine.onStatusChange(engine.getStatus());
            }
        }
        //this callback is called once the engine is no more
        thisPage.onEngineLost = function() {
           /// document.statusImg.src = that.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::statusImgDir + "status_waiting.png";
            document.getElementById("statusDiv").innerHTML = "Waiting for Push Engine creation...";
        }

        //bind the PushPage to the document 
        thisPage.bind();
        //create the engine
        thisPage.createEngine(this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::engineName, this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::libPath, "SHARE_SESSION", true);
    
    }-*/;
    
    //this method accepts a LSGWTNonVisualTable, extracts from it the contained
    //javascript NonVisualTable instance and sends such instance to the 
    //underlying PushPage instance.
    public void addTable(LSGWTNonOverWriteTable table, String id) {
        if (this.page == null) {
            throw new IllegalStateException("LSGWTPushPage not yet started");
        }
        this.addTable(table.getNVT(), id);
    }

    private native void addTable(JavaScriptObject table, String id) /*-{
        this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::page.addTable(table,id);
    }-*/;
    
    private native void initStatusHandler() /*-{
        // prefetch the images (especially useful for mobile devices)
        var prefetcher = new Array(9);
        for (i = 0; i <= 9; i++) {
          prefetcher[i] = new Image();
        }
//        prefetcher[0].src = this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::statusImgDir + "status_waiting.png";
//        prefetcher[1].src = this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::statusImgDir + "status_disconnected.png";
//        prefetcher[2].src = this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::statusImgDir + "status_disconnected_master.png";
//        prefetcher[3].src = this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::statusImgDir + "status_connected_streaming_master.png";
//        prefetcher[4].src = this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::statusImgDir + "status_connected_polling_master.png";
//        prefetcher[5].src = this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::statusImgDir + "status_connected_streaming.png";
//        prefetcher[6].src = this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::statusImgDir + "status_connected_polling.png";
//        prefetcher[7].src = this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::statusImgDir + "status_stalled.png";
//        prefetcher[8].src = this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::statusImgDir + "status_stalled_master.png";
//    
//        // prepare the HTML snippets to write to the page
//        var imgHtml1 = "<div style='position: ";
//        var imgHtml2 = "top: 0px; left: 0px;' onmouseover='document.getElementById(\"statusDiv\").style.visibility=\"visible\"' onmouseout='document.getElementById(\"statusDiv\").style.visibility=\"hidden\"'><img name='statusImg' src='" + this.@com.delta.acs.snapp.gates.web.client.lightstreamer.setup.LSGWTPushPage::statusImgDir + "status_waiting.png' /></div>";
//        var textHtml1 = "<div id='statusDiv' style='position: ";
//        var textHtml2 = "top: 10px; left: 20px; visibility: hidden; border: 1px solid #C0C0C0; padding: 3px; margin: 0px; font-family: Arial, Helvetica, sans-serif; font-size: 10px; color: #333333; background-color: #FFFFFF; filter:alpha(opacity=90); -moz-opacity:0.9; -khtml-opacity: 0.9; opacity: 0.9;'>Waiting for Push Engine creation...</div>";
//    
//        // create a container div element to host the other divs defined below
//        var statusContainer = document.createElement("div");
//        document.getElementsByTagName("body")[0].appendChild(statusContainer);
//        // create the div elements to host the status image and text and
//        // position them "absolute" or "fixed" based on the browser type
//        // IE < 7
//        statusContainer.innerHTML = "<!--[if lt IE 7]>" + imgHtml1 + "absolute; " + imgHtml2 + textHtml1 + "absolute; " + textHtml2 + "<![endif]-->";
//        // IE >= 7 and all other browsers
//        if (!document.statusImg) {
//          statusContainer.innerHTML = imgHtml1 + "fixed; " + imgHtml2 + textHtml1 + "fixed; " + textHtml2;
//        }
        
    }-*/;

    public void setDomain(String domain) {
        this.domain = domain;
    }

    public void setHost(String host) {
        this.host = host;
    }

    public void setPort(String port) {
        this.port = port;
    }

    public void setAdapterSet(String adapterSet) {
        this.adapterSet = adapterSet;
    }

    public void setLibPath(String libPath) {
        this.libPath = libPath;
    }

    public void setEngineName(String engineName) {
        this.engineName = engineName;
    }
    
    public void setStatusImgDir(String statusImgDir) {
        this.statusImgDir = statusImgDir;
    }
    
    public void setEngineKind(String engineKind) {
        this.engineKind = engineKind!=null ? engineKind : "SHARE_SESSION";
    }
    
    public void enableDebugAlerts(boolean debugAlerts) {
        this.debugAlerts = debugAlerts;
    }

    public void enableRemoteAlerts(boolean remoteAlerts) {
        this.remoteAlerts = remoteAlerts;
    }

}
